﻿// TextAtAngles (c) 2009 by Charles Petzold
using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media.Animation;

namespace TextAtAngles
{
    public partial class MainPage : UserControl
    {
        List<Storyboard> storyboards = new List<Storyboard>();

        public MainPage()
        {
            InitializeComponent();

            // Get all the TextBlock sizes
            // Note: all Height properties should be the same!
            Size bigSize = new Size(10000, 10000);
            txt1.Measure(bigSize);
            Size size1 = txt1.DesiredSize;
            txt2.Measure(bigSize);
            Size size2 = txt2.DesiredSize;
            txt3.Measure(bigSize);
            Size size3 = txt3.DesiredSize;

            txt1.BaseRectangle = new Rect(new Point(), size1);
            txt2.BaseRectangle = new Rect(new Point(), size2);
            txt3.BaseRectangle = new Rect(new Point(), size3);

            // These are the points for a normal display
            Point[,] ptBase = new Point[4, 2];   // Far left, left, right, far right x top and bottom

            ptBase[0, 0] = new Point(0, 0);
            ptBase[1, 0] = new Point(size1.Width, 0);
            ptBase[2, 0] = new Point(size1.Width + size2.Width, 0);
            ptBase[3, 0] = new Point(size1.Width + size2.Width + size3.Width, 0);

            ptBase[0, 1] = new Point(0, size1.Height);
            ptBase[1, 1] = new Point(size1.Width, size1.Height);
            ptBase[2, 1] = new Point(size1.Width + size2.Width, size1.Height);
            ptBase[3, 1] = new Point(size1.Width + size2.Width + size3.Width, size1.Height);

            CreateStoryboard(ptBase, CreatePointsForAnimation(ptBase, 0.67, 0.67, 0.67, 0.67, 0.01, 2.00, 0.01, 2.00));
            CreateStoryboard(ptBase, CreatePointsForAnimation(ptBase, 1.50, 1.50, 1.50, 1.50, 0.01, 0.75, 0.01, 0.75));
            CreateStoryboard(ptBase, CreatePointsForAnimation(ptBase, 1.00, 1.00, 1.00, 1.00, 0.01, 0.75, 0.01, 2.00));
            CreateStoryboard(ptBase, CreatePointsForAnimation(ptBase, 1.00, 1.00, 1.00, 1.00, 0.01, 2.00, 0.01, 0.75));
            CreateStoryboard(ptBase, CreatePointsForAnimation(ptBase, 0.75, 0.67, 0.75, 1.50, 0.50, 1.00, 0.50, 1.00));
            CreateStoryboard(ptBase, CreatePointsForAnimation(ptBase, 0.75, 1.50, 0.75, 0.67, 0.50, 1.00, 0.50, 1.00));

            // Start it going
            storyboards[0].Begin();
        }

        Point[,] CreatePointsForAnimation(Point[,] ptBase, double MLxSc, double MLySc,  // Scale: Middle text, left side
                                                           double MRxSc, double MRySc,  // Scale: Middle text, right side
                                                           double LxSc, double LySc,    // Scale: Left text (left side)
                                                           double RxSc, double RySc)    // Scale: Right text (right side)
        {
            Point[,] ptAnima = new Point[4, 2];

            ptAnima[1, 0] = new Point(MLxSc * ptBase[1, 0].X + (1 - MLxSc) * ptBase[2, 0].X,
                                      MLySc * ptBase[1, 0].Y + (1 - MLySc) * ptBase[1, 1].Y);

            ptAnima[1, 1] = new Point(MLxSc * ptBase[1, 1].X + (1 - MLxSc) * ptBase[2, 1].X,
                                      MLySc * ptBase[1, 1].Y + (1 - MLySc) * ptBase[1, 0].Y);

            ptAnima[2, 0] = new Point(MRxSc * ptBase[2, 0].X + (1 - MRxSc) * ptBase[1, 0].X,
                                      MRySc * ptBase[2, 0].Y + (1 - MRySc) * ptBase[2, 1].Y);

            ptAnima[2, 1] = new Point(MRxSc * ptBase[2, 1].X + (1 - MRxSc) * ptBase[1, 1].X,
                                      MRySc * ptBase[2, 1].Y + (1 - MRySc) * ptBase[2, 0].Y);

            ptAnima[0, 0] = new Point(LxSc * ptBase[0, 0].X + (1 - LxSc) * ptAnima[1, 0].X,
                                      LySc * ptBase[0, 0].Y + (1 - LySc) * ptBase[0, 1].Y);

            ptAnima[0, 1] = new Point(LxSc * ptBase[0, 1].X + (1 - LxSc) * ptAnima[1, 1].X,
                                      LySc * ptBase[0, 1].Y + (1 - LySc) * ptBase[0, 0].Y);

            ptAnima[3, 0] = new Point(RxSc * ptBase[3, 0].X + (1 - RxSc) * ptAnima[2, 0].X,
                                      RySc * ptBase[3, 0].Y + (1 - RySc) * ptBase[3, 1].Y);

            ptAnima[3, 1] = new Point(RxSc * ptBase[3, 1].X + (1 - RxSc) * ptAnima[2, 1].X,
                                      RySc * ptBase[3, 1].Y + (1 - RySc) * ptBase[3, 0].Y);

            return ptAnima;
        }

        void CreateStoryboard(Point[,] ptBase, Point[,] ptAnima)
        {
            Storyboard storyboard = new Storyboard();

            CreateAnimation(storyboard, ptBase, ptAnima, 0, 0, txt1, PerspectiveControl.TopLeftProperty);
            CreateAnimation(storyboard, ptBase, ptAnima, 0, 1, txt1, PerspectiveControl.BottomLeftProperty);
            CreateAnimation(storyboard, ptBase, ptAnima, 1, 0, txt1, PerspectiveControl.TopRightProperty);
            CreateAnimation(storyboard, ptBase, ptAnima, 1, 1, txt1, PerspectiveControl.BottomRightProperty);

            CreateAnimation(storyboard, ptBase, ptAnima, 1, 0, txt2, PerspectiveControl.TopLeftProperty);
            CreateAnimation(storyboard, ptBase, ptAnima, 2, 0, txt2, PerspectiveControl.TopRightProperty);
            CreateAnimation(storyboard, ptBase, ptAnima, 1, 1, txt2, PerspectiveControl.BottomLeftProperty);
            CreateAnimation(storyboard, ptBase, ptAnima, 2, 1, txt2, PerspectiveControl.BottomRightProperty);

            CreateAnimation(storyboard, ptBase, ptAnima, 2, 0, txt3, PerspectiveControl.TopLeftProperty);
            CreateAnimation(storyboard, ptBase, ptAnima, 2, 1, txt3, PerspectiveControl.BottomLeftProperty);
            CreateAnimation(storyboard, ptBase, ptAnima, 3, 0, txt3, PerspectiveControl.TopRightProperty);
            CreateAnimation(storyboard, ptBase, ptAnima, 3, 1, txt3, PerspectiveControl.BottomRightProperty);

            storyboard.Completed += new EventHandler(OnStoryboardCompleted);
            storyboards.Add(storyboard);
        }

        void CreateAnimation(Storyboard storyboard, Point[,] ptBase, Point[,] ptAnima, int corner, int topbot, 
                             DependencyObject target, DependencyProperty property)
        {
            PointAnimation anima = new PointAnimation()
            {
                From = ptBase[corner, topbot],
                To = ptAnima[corner, topbot],
                Duration = new Duration(TimeSpan.FromSeconds(3)),
                AutoReverse = true,
                FillBehavior = FillBehavior.Stop,
                EasingFunction = new SineEase()
                {
                    EasingMode = EasingMode.EaseInOut
                }
            };
            Storyboard.SetTarget(anima, target);
            Storyboard.SetTargetProperty(anima, new PropertyPath(property));
            storyboard.Children.Add(anima);
        }

        void OnStoryboardCompleted(object sender, EventArgs args)
        {
            Storyboard storyboard = sender as Storyboard;
            int index = (storyboards.IndexOf(storyboard) + 1) % storyboards.Count;
            storyboards[index].Begin();
        }
    }
}
